@@ -19,11 +19,11 @@ func (ui *tuiWindow) getTextDrawer() *textDrawer {
 	return &textDrawer{region: ui.region, screen: ui.screen}
 }
 
-func (ui *tuiWindow) setCursor(line int, offset int) {
+func (ui *tuiWindow) setCursor(line, offset int) {
 	ui.screen.ShowCursor(ui.region.left+offset, ui.region.top+line)
 }
 
-func (ui *tuiWindow) offsetStyleWidth(s *state.WindowState) int {
+func offsetStyleWidth(s *state.WindowState) int {
 	threshold := int64(0xfffff)
 	for i := range 10 {
 		if s.Length <= threshold {
@@ -38,7 +38,7 @@ func (ui *tuiWindow) drawWindow(s *state.WindowState, active bool) {
 	height, width := ui.region.height-2, s.Width
 	cursorPos := int(s.Cursor - s.Offset)
 	cursorLine := cursorPos / width
-	offsetStyleWidth := ui.offsetStyleWidth(s)
+	offsetStyleWidth := offsetStyleWidth(s)
 	eis := s.EditedIndices
 	for 0 < len(eis) && eis[1] <= s.Offset {
 		eis = eis[2:]
@@ -48,7 +48,10 @@ func (ui *tuiWindow) drawWindow(s *state.WindowState, active bool) {
 	var k int
 	for i := range height {
 		d.addTop(1).setLeft(0).setOffset(0)
-		d.setString(fmt.Sprintf(" %0*x", offsetStyleWidth, s.Offset+int64(i*width)), tcell.StyleDefault.Bold(i == cursorLine))
+		d.setString(
+			fmt.Sprintf(" %0*x", offsetStyleWidth, s.Offset+int64(i*width)),
+			tcell.StyleDefault.Bold(i == cursorLine),
+		)
 		d.setLeft(offsetStyleWidth + 3)
 		for j := range width {
 			b, style := byte(0), tcell.StyleDefault
@@ -134,7 +137,7 @@ func (ui *tuiWindow) drawHeader(s *state.WindowState, offsetStyleWidth int) {
 	}
 }
 
-func (ui *tuiWindow) drawScrollBar(s *state.WindowState, height int, left int) {
+func (ui *tuiWindow) drawScrollBar(s *state.WindowState, height, left int) {
 	stateSize := s.Size
 	if s.Cursor+1 == s.Length && s.Cursor == s.Offset+int64(s.Size) {
 		stateSize++
