@@ -14,13 +14,41 @@ import (
 	"github.com/itchyny/bed/mode"
 )
 
+func createTemp(dir, contents string) (*os.File, error) {
+	f, err := os.CreateTemp(dir, "")
+	if err != nil {
+		return nil, err
+	}
+	if _, err = f.WriteString(contents); err != nil {
+		return nil, err
+	}
+	if err = f.Close(); err != nil {
+		return nil, err
+	}
+	return f, nil
+}
+
 func TestManagerOpenEmpty(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
+	go func() {
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		ev := <-eventCh
+		if ev.Type != event.Error {
+			t.Errorf("event type should be %d but got: %d", event.Error, ev.Type)
+		}
+		if expected := "no file name"; ev.Error.Error() != expected {
+			t.Errorf("err should be %q but got: %v", expected, ev.Error)
+		}
+	}()
 	wm.SetSize(110, 20)
 	if err := wm.Open(""); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	windowStates, _, windowIndex, err := wm.State()
 	ws := windowStates[0]
@@ -45,35 +73,22 @@ func TestManagerOpenEmpty(t *testing.T) {
 	if err != nil {
 		t.Errorf("err should be nil but got: %v", err)
 	}
+	wm.Emit(event.Event{Type: event.Write})
+	<-waitCh
 	wm.Close()
 }
 
 func TestManagerOpenStates(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
-	wm.Init(eventCh, redrawCh)
+	wm.Init(nil, nil)
 	wm.SetSize(110, 20)
-	f, err := os.CreateTemp("", "bed-test-manager-open")
-	if err != nil {
-		t.Errorf("err should be nil but got %v", err)
-	}
 	str := "Hello, world! こんにちは、世界！"
-	n, err := f.WriteString(str)
-	if n != 41 {
-		t.Errorf("WriteString should return %d but got %d", 41, n)
-	}
+	f, err := createTemp(t.TempDir(), str)
 	if err != nil {
-		t.Errorf("err should be nil but got %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
-	if err != nil {
-		t.Errorf("err should be nil but got: %v", err)
-	}
-	if err := f.Close(); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
-	}
-	defer os.Remove(f.Name())
 	if err := wm.Open(f.Name()); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	windowStates, _, windowIndex, err := wm.State()
 	ws := windowStates[0]
@@ -103,23 +118,25 @@ func TestManagerOpenStates(t *testing.T) {
 
 func TestManagerOpenNonExistsWrite(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
 	go func() {
-		for {
-			select {
-			case <-eventCh:
-			case <-redrawCh:
-			}
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		for range 16 {
+			<-redrawCh
+		}
+		if ev := <-eventCh; ev.Type != event.QuitAll {
+			t.Errorf("event type should be %d but got: %d", event.QuitAll, ev.Type)
 		}
 	}()
 	wm.SetSize(110, 20)
-	f, _ := os.CreateTemp("", "bed-test-manager-open")
-	_ = f.Close()
-	_ = os.Remove(f.Name())
-	defer os.Remove(f.Name())
-	if err := wm.Open(f.Name()); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+	fname := filepath.Join(t.TempDir(), "test")
+	if err := wm.Open(fname); err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	_, _, _, _ = wm.State()
 	str := "Hello, world!"
@@ -129,13 +146,13 @@ func TestManagerOpenNonExistsWrite(t *testing.T) {
 		wm.Emit(event.Event{Type: event.Rune, Rune: c, Mode: mode.Insert})
 	}
 	wm.Emit(event.Event{Type: event.ExitInsert})
-	wm.Emit(event.Event{Type: event.Write})
+	wm.Emit(event.Event{Type: event.WriteQuit})
 	windowStates, _, windowIndex, err := wm.State()
 	ws := windowStates[0]
 	if windowIndex != 0 {
 		t.Errorf("window index should be %d but got %d", 0, windowIndex)
 	}
-	if expected := filepath.Base(f.Name()); ws.Name != expected {
+	if expected := filepath.Base(fname); ws.Name != expected {
 		t.Errorf("name should be %q but got %q", expected, ws.Name)
 	}
 	if ws.Width != 16 {
@@ -153,32 +170,32 @@ func TestManagerOpenNonExistsWrite(t *testing.T) {
 	if err != nil {
 		t.Errorf("err should be nil but got: %v", err)
 	}
-	bs, err := os.ReadFile(f.Name())
+	bs, err := os.ReadFile(fname)
 	if err != nil {
 		t.Errorf("err should be nil but got: %v", err)
 	}
 	if string(bs) != str {
 		t.Errorf("file contents should be %q but got %q", str, string(bs))
 	}
+	<-waitCh
 	wm.Close()
 }
 
 func TestManagerOpenExpandBacktick(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
-	wm.Init(eventCh, redrawCh)
+	wm.Init(nil, nil)
 	wm.SetSize(110, 20)
-	cmd, filename := "`which ls`", "ls"
+	cmd, name := "`which ls`", "ls"
 	if runtime.GOOS == "windows" {
-		cmd, filename = "`where ping`", "PING.EXE"
+		cmd, name = "`where ping`", "PING.EXE"
 	}
 	if err := wm.Open(cmd); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	windowStates, _, _, err := wm.State()
 	ws := windowStates[0]
-	if ws.Name != filename {
-		t.Errorf("name should be %q but got %q", filename, ws.Name)
+	if ws.Name != name {
+		t.Errorf("name should be %q but got %q", name, ws.Name)
 	}
 	if ws.Width != 16 {
 		t.Errorf("width should be %d but got %d", 16, ws.Width)
@@ -195,14 +212,145 @@ func TestManagerOpenExpandBacktick(t *testing.T) {
 	wm.Close()
 }
 
+func TestManagerOpenExpandHomedir(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("skip on Windows")
+	}
+	wm := NewManager()
+	wm.Init(nil, nil)
+	wm.SetSize(110, 20)
+	str := "Hello, world!"
+	f, err := createTemp(t.TempDir(), str)
+	if err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
+	}
+	home := os.Getenv("HOME")
+	t.Cleanup(func() {
+		if err := os.Setenv("HOME", home); err != nil {
+			t.Errorf("err should be nil but got: %v", err)
+		}
+	})
+	if err := os.Setenv("HOME", filepath.Dir(f.Name())); err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
+	}
+	for i, prefix := range []string{"~/", "$HOME/"} {
+		if err := wm.Open(prefix + filepath.Base(f.Name())); err != nil {
+			t.Fatalf("err should be nil but got: %v", err)
+		}
+		windowStates, _, windowIndex, err := wm.State()
+		ws := windowStates[i]
+		if windowIndex != i {
+			t.Errorf("window index should be %d but got %d", i, windowIndex)
+		}
+		if expected := filepath.Base(f.Name()); ws.Name != expected {
+			t.Errorf("name should be %q but got %q", expected, ws.Name)
+		}
+		if !strings.HasPrefix(string(ws.Bytes), str) {
+			t.Errorf("Bytes should start with %q but got %q", str, string(ws.Bytes))
+		}
+		if err != nil {
+			t.Errorf("err should be nil but got: %v", err)
+		}
+	}
+	wm.Close()
+}
+
+func TestManagerOpenChdirWrite(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("skip on Windows")
+	}
+	wm := NewManager()
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
+	wm.Init(eventCh, redrawCh)
+	f, err := createTemp(t.TempDir(), "Hello")
+	if err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
+	}
+	go func() {
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		ev := <-eventCh
+		if ev.Type != event.Info {
+			t.Errorf("event type should be %d but got: %d", event.Info, ev.Type)
+		}
+		dir, err := filepath.EvalSymlinks(filepath.Dir(f.Name()))
+		if err != nil {
+			t.Errorf("err should be nil but got: %v", err)
+		}
+		if expected := dir; ev.Error.Error() != expected {
+			t.Errorf("err should be %q but got: %v", expected, ev.Error)
+		}
+		ev = <-eventCh
+		if ev.Type != event.Info {
+			t.Errorf("event type should be %d but got: %d", event.Info, ev.Type)
+		}
+		if expected := filepath.Dir(dir); ev.Error.Error() != expected {
+			t.Errorf("err should be %q but got: %v", expected, ev.Error)
+		}
+		for range 11 {
+			<-redrawCh
+		}
+		ev = <-eventCh
+		if ev.Type != event.Info {
+			t.Errorf("event type should be %d but got: %d", event.Info, ev.Type)
+		}
+		if expected := "13 (0xd) bytes written"; !strings.HasSuffix(ev.Error.Error(), expected) {
+			t.Errorf("err should be %q but got: %v", expected, ev.Error)
+		}
+	}()
+	wm.SetSize(110, 20)
+	wm.Emit(event.Event{Type: event.Chdir, Arg: filepath.Dir(f.Name())})
+	if err := wm.Open(filepath.Base(f.Name())); err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
+	}
+	_, _, windowIndex, _ := wm.State()
+	if expected := 0; windowIndex != expected {
+		t.Errorf("windowIndex should be %d but got %d", expected, windowIndex)
+	}
+	wm.Emit(event.Event{Type: event.Chdir, Arg: "../"})
+	wm.Emit(event.Event{Type: event.StartAppendEnd})
+	wm.Emit(event.Event{Type: event.SwitchFocus})
+	for _, c := range ", world!" {
+		wm.Emit(event.Event{Type: event.Rune, Rune: c, Mode: mode.Insert})
+	}
+	wm.Emit(event.Event{Type: event.ExitInsert})
+	wm.Emit(event.Event{Type: event.Write})
+	bs, err := os.ReadFile(f.Name())
+	if err != nil {
+		t.Errorf("err should be nil but got: %v", err)
+	}
+	if expected := "Hello, world!"; string(bs) != expected {
+		t.Errorf("file contents should be %q but got %q", expected, string(bs))
+	}
+	<-waitCh
+	wm.Close()
+}
+
+func TestManagerOpenDirectory(t *testing.T) {
+	wm := NewManager()
+	wm.Init(nil, nil)
+	wm.SetSize(110, 20)
+	dir := t.TempDir()
+	if err := wm.Open(dir); err != nil {
+		if expected := dir + " is a directory"; err.Error() != expected {
+			t.Errorf("err should be %q but got: %v", expected, err)
+		}
+	} else {
+		t.Errorf("err should not be nil but got: %v", err)
+	}
+	wm.Close()
+}
+
 func TestManagerRead(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
-	wm.Init(eventCh, redrawCh)
+	wm.Init(nil, nil)
 	wm.SetSize(110, 20)
 	r := strings.NewReader("Hello, world!")
 	if err := wm.Read(r); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	windowStates, _, windowIndex, err := wm.State()
 	ws := windowStates[0]
@@ -229,19 +377,21 @@ func TestManagerRead(t *testing.T) {
 
 func TestManagerOnly(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
 	go func() {
-		for {
-			select {
-			case <-eventCh:
-			case <-redrawCh:
-			}
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		for range 4 {
+			<-eventCh
 		}
 	}()
 	wm.SetSize(110, 20)
 	if err := wm.Open(""); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	wm.Emit(event.Event{Type: event.Vnew})
 	wm.Emit(event.Event{Type: event.Vnew})
@@ -263,28 +413,34 @@ func TestManagerOnly(t *testing.T) {
 		t.Errorf("layout should be %#v but got %#v", expected, got)
 	}
 
+	<-waitCh
 	wm.Close()
 }
 
 func TestManagerAlternative(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
 	go func() {
-		for {
-			select {
-			case <-eventCh:
-			case <-redrawCh:
-			}
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		for range 9 {
+			<-eventCh
 		}
 	}()
 	wm.SetSize(110, 20)
 
+	if err := os.Chdir(os.TempDir()); err != nil {
+		t.Fatalf("err should be nil but got: %v", err)
+	}
 	if err := wm.Open("bed-test-manager-alternative-1"); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	if err := wm.Open("bed-test-manager-alternative-2"); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	wm.Emit(event.Event{Type: event.Alternative})
 	_, _, windowIndex, _ := wm.State()
@@ -356,24 +512,27 @@ func TestManagerAlternative(t *testing.T) {
 		t.Errorf("windowIndex should be %d but got %d", expected, windowIndex)
 	}
 
+	<-waitCh
 	wm.Close()
 }
 
 func TestManagerWincmd(t *testing.T) {
 	wm := NewManager()
-	eventCh, redrawCh := make(chan event.Event), make(chan struct{})
+	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
 	go func() {
-		for {
-			select {
-			case <-eventCh:
-			case <-redrawCh:
-			}
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
+		for range 17 {
+			<-eventCh
 		}
 	}()
 	wm.SetSize(110, 20)
 	if err := wm.Open(""); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	wm.Emit(event.Event{Type: event.Wincmd, Arg: "n"})
 	wm.Emit(event.Event{Type: event.Wincmd, Arg: "n"})
@@ -419,6 +578,7 @@ func TestManagerWincmd(t *testing.T) {
 		t.Errorf("layout should be %#v but got %#v", expected, got)
 	}
 
+	<-waitCh
 	wm.Close()
 }
 
@@ -426,25 +586,22 @@ func TestManagerCopyCutPaste(t *testing.T) {
 	wm := NewManager()
 	eventCh, redrawCh, waitCh := make(chan event.Event), make(chan struct{}), make(chan struct{})
 	wm.Init(eventCh, redrawCh)
-	f, err := os.CreateTemp("", "bed-test-manager-copy-cut-paste")
-	if err != nil {
-		t.Errorf("err should be nil but got %v", err)
-	}
 	str := "Hello, world!"
-	_, err = f.WriteString(str)
+	f, err := createTemp(t.TempDir(), str)
 	if err != nil {
-		t.Errorf("err should be nil but got %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
-	if err := f.Close(); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
-	}
-	defer os.Remove(f.Name())
 	wm.SetSize(110, 20)
 	if err := wm.Open(f.Name()); err != nil {
-		t.Errorf("err should be nil but got: %v", err)
+		t.Fatalf("err should be nil but got: %v", err)
 	}
 	_, _, _, _ = wm.State()
 	go func() {
+		defer func() {
+			close(eventCh)
+			close(redrawCh)
+			close(waitCh)
+		}()
 		<-redrawCh
 		<-redrawCh
 		<-redrawCh
@@ -509,7 +666,6 @@ func TestManagerCopyCutPaste(t *testing.T) {
 		if expected := "Hefoobarfoobarfoobarlrld!"; !strings.HasPrefix(string(ws.Bytes), expected) {
 			t.Errorf("Bytes should start with %q but got %q", expected, string(ws.Bytes))
 		}
-		close(waitCh)
 	}()
 	wm.Emit(event.Event{Type: event.CursorNext, Mode: mode.Normal, Count: 3})
 	wm.Emit(event.Event{Type: event.StartVisual})
