@@ -1,11 +1,13 @@
 package window
 
 import (
+	"cmp"
 	"errors"
 	"fmt"
 	"io"
 	"strconv"
 	"sync"
+	"unicode"
 	"unicode/utf8"
 
 	"github.com/itchyny/bed/buffer"
@@ -25,7 +27,7 @@ type window struct {
 	history          *history.History
 	searcher         *searcher.Searcher
 	searchTick       uint64
-	filename         string
+	path             string
 	name             string
 	height           int64
 	width            int64
@@ -57,7 +59,10 @@ type readAtSeeker interface {
 	io.Seeker
 }
 
-func newWindow(r readAtSeeker, filename string, name string, eventCh chan<- event.Event, redrawCh chan<- struct{}) (*window, error) {
+func newWindow(
+	r readAtSeeker, path, name string,
+	eventCh chan<- event.Event, redrawCh chan<- struct{},
+) (*window, error) {
 	buffer := buffer.NewBuffer(r)
 	length, err := buffer.Len()
 	if err != nil {
@@ -69,7 +74,7 @@ func newWindow(r readAtSeeker, filename string, name string, eventCh chan<- even
 		buffer:      buffer,
 		history:     history,
 		searcher:    searcher.NewSearcher(r),
-		filename:    filename,
+		path:        path,
 		name:        name,
 		length:      length,
 		visualStart: -1,
@@ -289,66 +294,48 @@ func (w *window) readBytes(offset int64, l int) (int, []byte, error) {
 func (w *window) writeTo(r *event.Range, dst io.Writer) (int64, error) {
 	w.mu.Lock()
 	defer w.mu.Unlock()
+	var from, to int64
 	if r == nil {
-		if _, err := w.buffer.Seek(0, io.SeekStart); err != nil {
+		from, to = 0, w.length-1
+	} else {
+		var err error
+		if from, err = w.positionToOffset(r.From); err != nil {
 			return 0, err
 		}
-		return io.Copy(dst, w.buffer)
+		if to, err = w.positionToOffset(r.To); err != nil {
+			return 0, err
+		}
+		if from > to {
+			from, to = to, from
+		}
 	}
-	var from, to int64
-	var err error
-	if from, err = w.positionToOffset(r.From); err != nil {
-		return 0, err
-	}
-	if to, err = w.positionToOffset(r.To); err != nil {
-		return 0, err
-	}
-	if from > to {
-		from, to = to, from
-	}
-	if _, err := w.buffer.Seek(from, io.SeekStart); err != nil {
-		return 0, err
-	}
-	return io.Copy(dst, io.LimitReader(w.buffer, to-from+1))
+	return io.Copy(dst, io.NewSectionReader(w.buffer, from, to-from+1))
 }
 
 func (w *window) positionToOffset(pos event.Position) (int64, error) {
+	var offset int64
 	switch pos := pos.(type) {
 	case event.Absolute:
-		return max(
-			min(pos.Offset, max(w.length, 1)-1),
-			0,
-		), nil
+		offset = pos.Offset
 	case event.Relative:
-		return w.cursor + max(
-			min(pos.Offset, max(w.length, 1)-1-w.cursor),
-			-w.cursor,
-		), nil
+		offset = w.cursor + pos.Offset
 	case event.End:
-		return max(w.length, 1) - 1 + max(
-			min(pos.Offset, 0),
-			-(max(w.length, 1)-1),
-		), nil
+		offset = max(w.length, 1) - 1 + pos.Offset
 	case event.VisualStart:
 		if w.visualStart < 0 {
 			return 0, errors.New("no visual selection found")
 		}
-		// TODO: save visualStart after exitting visual mode
-		return w.visualStart + max(
-			min(pos.Offset, max(w.length, 1)-1-w.visualStart),
-			-w.visualStart,
-		), nil
+		// TODO: save visualStart after exiting visual mode
+		offset = w.visualStart + pos.Offset
 	case event.VisualEnd:
 		if w.visualStart < 0 {
 			return 0, errors.New("no visual selection found")
 		}
-		return w.cursor + max(
-			min(pos.Offset, max(w.length, 1)-1-w.cursor),
-			-w.cursor,
-		), nil
+		offset = w.cursor + pos.Offset
 	default:
 		return 0, errors.New("invalid range")
 	}
+	return max(min(offset, max(w.length, 1)-1), 0), nil
 }
 
 func (w *window) state(width, height int) (*state.WindowState, error) {
@@ -553,7 +540,7 @@ func (w *window) cursorGotoPos(pos event.Position, cmdName string) {
 		}
 	}
 	if offset, err := w.positionToOffset(pos); err == nil {
-		w.cursor = max(min(offset, max(w.length, 1)-1), 0)
+		w.cursor = offset
 		if w.cursor < w.offset {
 			w.offset = (max(w.cursor/w.width, w.height/2) - w.height/2) * w.width
 		} else if w.cursor >= w.offset+w.height*w.width {
@@ -700,43 +687,25 @@ func (w *window) windowBottom(count int64) {
 	w.cursor = (w.offset/w.width + h - min(h, max(count, 1)-1)) * w.width
 }
 
-func isDigit(b byte) bool {
-	return '\x30' <= b && b <= '\x39'
-}
-
-func isWhite(b byte) bool {
-	return b == '\x00' || b == '\x09' || b == '\x0a' || b == '\x0d' || b == '\x20'
-}
-
 func (w *window) jumpTo() {
-	s := 50
-	_, bytes, err := w.readBytes(max(w.cursor-int64(s), 0), 2*s)
+	i := min(w.cursor, 16)
+	_, bytes, err := w.readBytes(w.cursor-i, 32)
 	if err != nil {
 		return
 	}
-	var i, j int
-	for i = s; i < 2*s; i++ {
-		if isWhite(bytes[i]) {
+	for ; i >= 0; i-- {
+		if !unicode.IsDigit(rune(bytes[i])) {
+			bytes = bytes[i+1:]
 			break
 		}
 	}
-	if i == 2*s || !isDigit(bytes[i]) {
-		return
-	}
-	for ; i > 0; i-- {
-		if isDigit(bytes[i-1]) {
+	for i := 0; i < len(bytes); i++ {
+		if !unicode.IsDigit(rune(bytes[i])) {
+			bytes = bytes[:i]
 			break
 		}
 	}
-	for j = i; j < 2*s; j++ {
-		if isDigit(bytes[j]) {
-			break
-		}
-	}
-	if j == 2*s {
-		return
-	}
-	offset, _ := strconv.ParseInt(string(bytes[i:j]), 10, 64)
+	offset, _ := strconv.ParseInt(string(bytes), 10, 64)
 	if offset <= 0 || w.length <= offset {
 		return
 	}
@@ -1091,9 +1060,10 @@ func (w *window) abortSearch() {
 	}
 }
 
+func (w *window) setPathName(path, name string) {
+	w.path, w.name = path, name
+}
+
 func (w *window) getName() string {
-	if w.name != "" {
-		return w.name
-	}
-	return "[No Name]"
+	return cmp.Or(w.name, "[No Name]")
 }
